
// NOTICE: 

//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

/*

Original code by Trevlac.  The only information I have in terms of credit is:
http://www.trevlac.us/FilterDocs/

Brief description of features I added to the GUI panel

By "window", I mean what Trevlac refers to as "mask" - where the overlay video
will appear.

Size - computes the size of the selected window for you
Bilinear stretch - uses a better method of resizing video (interpolation)
window geometry box - type in window coordinates if you prefer
Keep Aspect - while selecting window, maintain aspect ratio of the overlay video
Set Frame Start/End and box - specify when (by frame #) the overlay video
   will start and possibly end in relation to the background video.  For example
   100 0 would start the overlay video when the background video reaches frame
   100 (0 end means end not specified)
Rev. - reverses the overlay video (can be slow)
Repeat video - repeat the overlay video when it reaches the end
Maintain black picture - keep up a black box (with border if specified, etc.)
   after the overlay video ends
Frame Off. - specify the starting frame of the overlay video
Border, size, and color - add a border to the overlay video window
Frame Cnt./Res. - shows the frame count and resolution of the overlay video
Shadow, size, percent, type, Fixed shift, color - add a shadow to the overlay
   video window.  percent controls the amount of background video that shows
   through the shadow.  Fixed shift can be negative, which would give a shadow
   border on the top and on the left side as well (in conjunction with the
   color, a glow effect can be achieved)

I also corrected a few bugs.

Mark Craig
M.C. Software
http://youtube.com/user/MrMcsoftware
http://sites.google.com/site/mrmcsoftware
http://mcsoftware.deviantart.com

*/


///////////////////////////////////////////////////////////////////////////
// HEADERS
///////////////////////////////////////////////////////////////////////////


#define VDEXT_VIDEO_FILTER
#define VDEXT_MAIN
#define PRGB(r,g,b)          ((COLORREF)(((BYTE)(b)|((WORD)((BYTE)(g))<<8))|(((DWORD)(BYTE)(r))<<16)))
#define PRGB2RGB(rgb)          ((COLORREF)(((BYTE)(GetBValue(rgb))|((WORD)((BYTE)(GetGValue(rgb)))<<8))|(((DWORD)(BYTE)(GetRValue(rgb)))<<16)))
#define RGB2PRGB(rgb)          ((COLORREF)(((BYTE)(GetBValue(rgb))|((WORD)((BYTE)(GetGValue(rgb)))<<8))|(((DWORD)(BYTE)(GetRValue(rgb)))<<16)))


#include <stdio.h>
#include <windows.h>
#include <commctrl.h>
#include <commdlg.h>
#include <vfw.h>
#include <time.h>
#include <math.h>

#include "resource.h"
#include "Filter.h"


//#include "..\include\StdAfx.h"
#include "ScriptInterpreter.h"
#include "ScriptError.h"
#include "ScriptValue.h"
#include "VBitmap.h"



///////////////////////////////////////////////////////////////////////////
// FUNCTION DEFS
///////////////////////////////////////////////////////////////////////////

int ezMaskRunProc(const FilterActivation *fa, const FilterFunctions *ff);
int ezMaskStartProc(FilterActivation *fa, const FilterFunctions *ff);
int ezMaskEndProc(FilterActivation *fa, const FilterFunctions *ff);
void deinitProc(FilterActivation *fa, const FilterFunctions *ff);

long ezMaskParamProc(FilterActivation *fa, const FilterFunctions *ff);
int ezMaskConfigProc(FilterActivation *fa, const FilterFunctions *ff, HWND hwnd);
void ezMaskStringProc(const FilterActivation *fa, const FilterFunctions *ff, char *str);
void ezMaskScriptConfig(IScriptInterpreter *isi, void *lpVoid, CScriptValue *argv, int argc);
bool ezMaskFssProc(FilterActivation *fa, const FilterFunctions *ff, char *buf, int buflen);
void xRectFill(const FilterActivation *fa, PixCoord x, PixCoord y, PixDim dx, PixDim dy, Pixel32 c);
static const char *openFileDlg(HWND hwnd, const char *oldfn);


LRESULT CALLBACK nPreviewWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);

extern "C" int __declspec(dllexport) __cdecl VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat);
extern "C" void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff);


///////////////////////////////////////////////////////////////////////////
// DATA DEFS
///////////////////////////////////////////////////////////////////////////

// Color typedef added by MARK
typedef struct Color
	{
	unsigned char b,g,r,a;
	} Color;

//****************************************
// Structure where we hold our Global 
// filter specific Data, like dialog settings,
// and lookup tables.
//
// (various variables added by MARK)
//
// This is passed into every function ....
//****************************************
typedef struct MyFilterData {
    bool     fExpand;
    bool     fThird;

	RECT rcMask;
	POINTS ptsMove;

	int dstW;
	int dstH;
	long frame;
	long saveFrame;
	int mode;
	int inverse;
	int maintbp;
	int drawing;
	int curframe;
	int setstart;
	int startframe;
	int endframe;
	int repeat;
	int length;
	int border;
	int bordersize;
	int aspect;
	int width;
	int height;
	int frameoff;
	int reverse;
	int bilinear;
	float power;
	int shadow;
	int samount;
	int spercent;
	int grad;
	int fshift;
	COLORREF srgb;
	COLORREF rgb;
	char szAviPath[MAX_PATH];

	HWND fpHwnd;
	HWND dlgHwnd;
	WNDPROC fpWndProc;
	IFilterPreview *ifp;
	FilterStateInfo *fsi;
    PAVIFILE pfile; 
    PAVISTREAM ppavi; 
	PGETFRAME pgf;
	LPBYTE pData;
	VBitmap *mySrc;
	HDC hdc2;
	int brb;

} MyFilterData;

MyFilterData *g_mfd;


void testAVIFile(char * filename, MyFilterData *mfd);

//****************************************
// ?????
//****************************************
ScriptFunctionDef ezMask_func_defs[]={
    { (ScriptFunctionPtr)ezMaskScriptConfig, "Config", "0iiii" },
    { NULL },
};


//****************************************
// ?????
//****************************************
CScriptObject ezMask_obj={
    NULL, ezMask_func_defs
};

//****************************************
// Structure that tells vDub about our filter
// Holds name, description, and pointers to
// the functions vdub calls for the filter
// to do it's thing.
//****************************************
struct FilterDefinition filterDef_ezMask = {

    NULL, NULL, NULL,       // next, prev, module
    "videoOverlay - Beta",             // name
    "Overlay videos",
                            // desc
    "trevlac & Mark Craig",               // maker
    NULL,                   // private_data
    sizeof(MyFilterData),   // inst_data_size

    NULL,                   // initProc
    deinitProc,        // deinitProc
    ezMaskRunProc,        // runProc
    ezMaskParamProc,      // paramProc
    ezMaskConfigProc,     // configProc
    ezMaskStringProc,     // stringProc
    ezMaskStartProc,      // startProc
    ezMaskEndProc,        // endProc

    &ezMask_obj,          // script_obj
    ezMaskFssProc,        // fssProc

};

static FilterDefinition *fd_ezMask;


///////////////////////////////////////////////////////////////////////////
// FUNCTIONS
///////////////////////////////////////////////////////////////////////////

// min/max/rnd/shadow added by MARK

int min(int v1,int v2)
{
int v;

v=v1;
if (v2<v1) { v=v2; }
return(v);
}

int max(int v1,int v2)
{
int v;

v=v1;
if (v2>v1) { v=v2; }
return(v);
}

float rnd(int r,MyFilterData *mfd)
{
float ran;

ran=(rand()/mfd->power)*(float)(r);
return(ran);
}

int shadow(int x,int y,int dx,int dy,int a,int gradi,int per,int p,int sh,const FilterActivation *fa)
{
MyFilterData *mfd=(MyFilterData *)fa->filter_data;
Pixel32 *src,*dst,*ps,*pd;
int xp,yp,xc,ycs,ycd,scol=1,grad=1;
float scf,sf,sfi;
Color *cols,*cold;

if ((dx<2)||(dy<2)) { return(0); }
x+=sh; y+=sh;
src=(Pixel32 *)fa->src.data;
dst=(Pixel32 *)fa->dst.data;
if (gradi<3) { p=a; }
if (mfd->srgb==0) { scol=0; }
scf=(float)(per)/100.0;
if ((gradi==0)||(gradi==5)) { grad=0; }
sfi=(1.0-scf)/(float)(a);
sf=scf;
for (yp=dy-p-1;yp>=0;yp--)
	{
	for (xp=0;xp<a;xp++)
		{
		if (grad)
			{
			if (yp<a)
				{
/* ? +1 */
				if (yp<(a-1-xp)) { sf=scf+((float)(a-1-yp+1))*(sfi); }
				else { sf=scf+(float)(xp)*(sfi); }
				}
			else { sf=scf+(float)(xp)*(sfi); }
			if ((gradi==1)||(gradi==3)) { sf-=(50.0-rnd(100,mfd))/600.0; if (sf<0) { sf=0; } else if (sf>1) { sf=1.0; } }
			}
		/* I think src and dst geometries, etc. will always be the same, */
		/* but in case they're not... */
		xc=(x+dx+xp); ycs=fa->src.h-1-(y+p+yp); ycd=fa->dst.h-1-(y+p+yp);
		if ((xc<0)||(xc>=fa->dst.w)||(ycd<0)||(ycd>=fa->dst.h)) continue;
		ps=src+(ycs)*(fa->src.w+fa->src.modulo)+(xc);
		pd=dst+(ycd)*(fa->dst.w+fa->dst.modulo)+(xc);
		cols=(Color *)ps; cold=(Color *)pd;
		cold->r=(unsigned char)((float)(cols->r)*sf);
		cold->g=(unsigned char)((float)(cols->g)*sf);
		cold->b=(unsigned char)((float)(cols->b)*sf);
		if (scol)
			{
			cold->r+=(unsigned char)((1.0-sf)*(float)(GetRValue(mfd->srgb)));
			cold->g+=(unsigned char)((1.0-sf)*(float)(GetGValue(mfd->srgb)));
			cold->b+=(unsigned char)((1.0-sf)*(float)(GetBValue(mfd->srgb)));
			}
		}
	}
sfi=(1.0-scf)/(float)(a);
for (yp=a-1;yp>=0;yp--)
	{
	for (xp=0;xp<dx-p+a;xp++)
		{
		if (grad)
			{
			if (xp<a)
				{
/* ? +1 */
				if (yp<(a-1-xp)) { sf=scf+((float)(a-1-xp+1))*(sfi); }
				else { sf=scf+(float)(yp)*(sfi); }
				}
			else if (xp>dx-p)
				{
				if (yp<((xp-dx+p))) { sf=scf+((float)((xp-dx+p)))*(sfi); }
				else { sf=scf+(float)(yp)*(sfi); }
				}
			else { sf=scf+(float)(yp)*(sfi); }
			if ((gradi==1)||(gradi==3)) { sf-=(50.0-rnd(100,mfd))/600.0; if (sf<0) { sf=0; } else if (sf>1) { sf=1.0; } }
			}
		xc=(x+p+xp); ycs=fa->src.h-1-(y+dy+yp); ycd=fa->dst.h-1-(y+dy+yp);
		if ((xc<0)||(xc>=fa->dst.w)||(ycd<0)||(ycd>=fa->dst.h)) continue;
		ps=src+(ycs)*(fa->src.w+fa->src.modulo)+(xc);
		pd=dst+(ycd)*(fa->dst.w+fa->dst.modulo)+(xc);
		cols=(Color *)ps; cold=(Color *)pd;
		cold->r=(unsigned char)((float)(cols->r)*sf);
		cold->g=(unsigned char)((float)(cols->g)*sf);
		cold->b=(unsigned char)((float)(cols->b)*sf);
		if (scol)
			{
			cold->r+=(unsigned char)((1.0-sf)*(float)(GetRValue(mfd->srgb)));
			cold->g+=(unsigned char)((1.0-sf)*(float)(GetGValue(mfd->srgb)));
			cold->b+=(unsigned char)((1.0-sf)*(float)(GetBValue(mfd->srgb)));
			}
		}
	}
if (p>=0) { return(1); }
for (yp=dy-p-1;yp>=0;yp--)
	{
	for (xp=0;xp<a;xp++)
		{
		if (grad)
			{
			if (yp<a)
				{
/* ? +1 */
				if (yp<(a-1-xp)) { sf=scf+((float)(a-1-yp+1))*(sfi); }
				else { sf=scf+(float)(xp)*(sfi); }
				}
			else { sf=scf+(float)(xp)*(sfi); }
			if ((gradi==1)||(gradi==3)) { sf-=(50.0-rnd(100,mfd))/600.0; if (sf<0) { sf=0; } else if (sf>1) { sf=1.0; } }
			}
		xc=(x+a-xp+p); ycs=fa->src.h-1-(y+p+yp); ycd=fa->dst.h-1-(y+p+yp);
		if ((xc<0)||(xc>=fa->dst.w)||(ycd<0)||(ycd>=fa->dst.h)) continue;
		ps=src+(ycs)*(fa->src.w+fa->src.modulo)+(xc);
		pd=dst+(ycd)*(fa->dst.w+fa->dst.modulo)+(xc);
		cols=(Color *)ps; cold=(Color *)pd;
		cold->r=(unsigned char)((float)(cols->r)*sf);
		cold->g=(unsigned char)((float)(cols->g)*sf);
		cold->b=(unsigned char)((float)(cols->b)*sf);
		if (scol)
			{
			cold->r+=(unsigned char)((1.0-sf)*(float)(GetRValue(mfd->srgb)));
			cold->g+=(unsigned char)((1.0-sf)*(float)(GetGValue(mfd->srgb)));
			cold->b+=(unsigned char)((1.0-sf)*(float)(GetBValue(mfd->srgb)));
			}
		}
	}
for (yp=a-1;yp>=0;yp--)
	{
	for (xp=0;xp<dx-p-a;xp++)
		{
		if (grad)
			{
			sf=scf+(float)(a-1-yp)*(sfi);
			if ((gradi==1)||(gradi==3)) { sf-=(50.0-rnd(100,mfd))/600.0; if (sf<0) { sf=0; } else if (sf>1) { sf=1.0; } }
			}
		xc=(x+p+xp+a); ycs=fa->src.h-1-(y+p+yp+1); ycd=fa->dst.h-1-(y+p+yp+1);
		if ((xc<0)||(xc>=fa->dst.w)||(ycd<0)||(ycd>=fa->dst.h)) continue;
		ps=src+(ycs)*(fa->src.w+fa->src.modulo)+(xc);
		pd=dst+(ycd)*(fa->dst.w+fa->dst.modulo)+(xc);
		cols=(Color *)ps; cold=(Color *)pd;
		cold->r=(unsigned char)((float)(cols->r)*sf);
		cold->g=(unsigned char)((float)(cols->g)*sf);
		cold->b=(unsigned char)((float)(cols->b)*sf);
		if (scol)
			{
			cold->r+=(unsigned char)((1.0-sf)*(float)(GetRValue(mfd->srgb)));
			cold->g+=(unsigned char)((1.0-sf)*(float)(GetGValue(mfd->srgb)));
			cold->b+=(unsigned char)((1.0-sf)*(float)(GetBValue(mfd->srgb)));
			}
		}
	}
return(1);
}

//****************************************
// VDub calls this exported function to 
// Add our filter to the list ..
//****************************************

int __declspec(dllexport) __cdecl VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat) {
    if (!(fd_ezMask = ff->addFilter(fm, &filterDef_ezMask, sizeof(FilterDefinition))))
        return 1;

	INITIALIZE_VTBLS;

	vdfd_ver = VIRTUALDUB_FILTERDEF_VERSION;
	vdfd_compat = VIRTUALDUB_FILTERDEF_COMPATIBLE;

    return 0;
}

//****************************************
// Exported function to remove us from the list
//****************************************
void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff) {
    ff->removeFilter(fd_ezMask);
}


//****************************************
// StartProc - Just before we start to filter
//
// Some additions/fixes by MARK
//
// Good place to .....
//****************************************
int ezMaskStartProc(FilterActivation *fa, const FilterFunctions *ff) {
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	if(mfd->ppavi)
		{
/* MARK - need to comment out to avoid crash ? */
		/* mfd->pgf = AVIStreamGetFrameOpen(mfd->ppavi,NULL); */
		mfd->curframe = 0;
		mfd->length=AVIStreamLength(mfd->ppavi);
		}

	mfd->mySrc = new VBitmap();
	mfd->power=(float)pow(2.0,15.0);
	srand((unsigned)time(NULL));
	rand();


    return 0;
}


//****************************************
// RunProc gets called for each frame.
//
// Some additions/fixes by MARK
//
// This is where the filtering is done.
//****************************************

int ezMaskRunProc(const FilterActivation *fa, const FilterFunctions *ff) {
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;
    PixDim w, h;
    Pixel32 *src, *dst;
    int shs,shw,shh;

	LPBITMAPINFOHEADER lpbih;
	

	RECT *rcMask, rcDraw;

	rcMask = &(mfd->rcMask);

	mfd->saveFrame = fa->pfsi->lCurrentFrame;
	mfd->curframe = mfd->saveFrame;
	if (mfd->setstart) { mfd->curframe-=mfd->startframe; }
	if ((mfd->repeat)&&(mfd->length!=0)) { mfd->curframe=(mfd->curframe)%(mfd->length-mfd->frameoff); }

    src = (Pixel32 *)fa->src.data;
    dst = (Pixel32 *)fa->dst.data;

	if(mfd->drawing) {

			h = fa->src.h;
			do {
				w = fa->src.w;
				do {
					*dst++ = *src++;
				} while(--w);

				src = (Pixel32 *)((char *)src + fa->src.modulo);
				dst = (Pixel32 *)((char *)dst + fa->dst.modulo);
			} while(--h);

			rcDraw.top = (rcMask->top > rcMask->bottom)?rcMask->bottom:rcMask->top;
			rcDraw.bottom = (rcMask->top < rcMask->bottom)?rcMask->bottom:rcMask->top;
			rcDraw.left = (rcMask->left < rcMask->right)?rcMask->left:rcMask->right;
			rcDraw.right = (rcMask->left > rcMask->right)?rcMask->left:rcMask->right;


			xRectFill(fa,rcDraw.left,rcDraw.top,rcDraw.right - rcDraw.left,2,0xffffff);
			xRectFill(fa,rcDraw.left,rcDraw.top,2,rcDraw.bottom - rcDraw.top,0xffffff);
			xRectFill(fa,rcDraw.right,rcDraw.top,2,rcDraw.bottom - rcDraw.top,0xffffff);
			xRectFill(fa,rcDraw.left,rcDraw.bottom,rcDraw.right - rcDraw.left,2,0xffffff);


	} else {

		//--- Move src to dst ---

		h = fa->src.h;

		do {
			w = fa->src.w;
			do { *dst++ = *src++;} while(--w);

			src = (Pixel32 *)((char *)src + fa->src.modulo);
			dst = (Pixel32 *)((char *)dst + fa->dst.modulo);
		} while(--h);


		//--- If we have a file ---
		if(mfd->pgf) {

			if ((mfd->setstart)&&((mfd->saveFrame<mfd->startframe)||((mfd->saveFrame>mfd->endframe)&&(mfd->endframe!=0)))) { return 0; }
			if (mfd->reverse) lpbih = (LPBITMAPINFOHEADER) AVIStreamGetFrame(mfd->pgf, mfd->length-(mfd->curframe+mfd->frameoff)-1);
			else lpbih = (LPBITMAPINFOHEADER) AVIStreamGetFrame(mfd->pgf, mfd->curframe+mfd->frameoff);
			if ((lpbih==NULL)&&(mfd->repeat))
				{
				mfd->curframe=0;
				if (mfd->reverse) lpbih = (LPBITMAPINFOHEADER) AVIStreamGetFrame(mfd->pgf, mfd->length-(mfd->curframe+mfd->frameoff)-1);
				else lpbih = (LPBITMAPINFOHEADER) AVIStreamGetFrame(mfd->pgf, mfd->curframe+mfd->frameoff);
				}
			mfd->curframe++;

			//--- if we have a Frame 
			if(lpbih) {
				if (mfd->shadow)
					{
					shs=mfd->border*mfd->bordersize-1;
					shw=rcMask->right-rcMask->left+1;
					shh=rcMask->bottom-rcMask->top+1;
					shadow(rcMask->left,rcMask->top,shw,shh,mfd->samount,mfd->grad,mfd->spercent,mfd->fshift,shs,fa);
					}
				if (mfd->border)
					fa->dst.RectFill(rcMask->left-mfd->bordersize,
						rcMask->top-mfd->bordersize,
						rcMask->right - rcMask->left + mfd->bordersize*2,
						rcMask->bottom - rcMask->top + mfd->bordersize*2,
						RGB2PRGB(mfd->rgb));

				//---- Make a vdub bitmap 
				mfd->pData = (LPBYTE)lpbih + lpbih->biSize;
				mfd->mySrc = &(mfd->mySrc->init(mfd->pData, lpbih));

				//--- If we are fit to frame
				if(mfd->inverse) {

					//--- If not 32 make it so --
					if(mfd->mySrc->depth !=32) {
						VBitmap *mySrc2;
						LPBYTE pData2;
						int w,h;

						mySrc2 = new VBitmap();
						w=mfd->mySrc->w;
						h=mfd->mySrc->h;
						pData2=(LPBYTE)calloc(1,w*h*4);
						if (pData2)
							{
							mySrc2 = &(mySrc2->init(pData2, w, h, 32));
							mySrc2->BitBlt(0,0,(VBitmap *)mfd->mySrc,0,0,-1,-1);

							if (mfd->bilinear)
								{
								fa->dst.StretchBltBilinearFast(rcMask->left,
									rcMask->top,rcMask->right - rcMask->left,
									rcMask->bottom - rcMask->top,mySrc2,0,0,
									mfd->mySrc->w,mfd->mySrc->h);
								}
							else
								{
								fa->dst.StretchBltNearestFast(rcMask->left,
									rcMask->top,rcMask->right - rcMask->left,
									rcMask->bottom - rcMask->top,mySrc2,0,0,
									mfd->mySrc->w,mfd->mySrc->h);
									/* MARK - was ,&(fa->src),0,0,fa->src.w,fa->src.h); */
								}
							free(pData2);
							}
						if (mySrc2) { delete mySrc2; }

					//---- Was 32 bit so mySrc to dst
					} else {

						if (mfd->bilinear)
							{
							fa->dst.StretchBltBilinearFast(rcMask->left,
								rcMask->top,rcMask->right - rcMask->left,
								rcMask->bottom - rcMask->top,mfd->mySrc,0,0,
								mfd->mySrc->w,mfd->mySrc->h);
							}
						else
							{
							fa->dst.StretchBltNearestFast(rcMask->left,
								rcMask->top,rcMask->right - rcMask->left,
								rcMask->bottom - rcMask->top,mfd->mySrc,0,0,
								mfd->mySrc->w,mfd->mySrc->h);
							}
					}

				//---- No stretch just blit
				} else {
					fa->dst.BitBlt(rcMask->left,rcMask->top,(VBitmap *)mfd->mySrc
						,0,0,rcMask->right - rcMask->left,rcMask->bottom - rcMask->top);
				}
			} //---- No frame ? don't do squat unless maintain black picture
			else
				if (mfd->maintbp)
					{
					if (mfd->shadow)
						{
						shs=mfd->border*mfd->bordersize-1;
						shw=rcMask->right-rcMask->left+1;
						shh=rcMask->bottom-rcMask->top+1;
						shadow(rcMask->left,rcMask->top,shw,shh,mfd->samount,mfd->grad,mfd->spercent,mfd->fshift,shs,fa);
						}
					if (mfd->border)
						fa->dst.RectFill(rcMask->left-mfd->bordersize,
							rcMask->top-mfd->bordersize,
							rcMask->right - rcMask->left + mfd->bordersize*2,
							rcMask->bottom - rcMask->top + mfd->bordersize*2,
							RGB2PRGB(mfd->rgb));
					fa->dst.RectFill(rcMask->left,rcMask->top
						,rcMask->right - rcMask->left,rcMask->bottom - rcMask->top,0);
					}

		//--- No file fill black
		} else {
			if (mfd->shadow)
				{
				shs=mfd->border*mfd->bordersize-1;
				shw=rcMask->right-rcMask->left+1;
				shh=rcMask->bottom-rcMask->top+1;
				shadow(rcMask->left,rcMask->top,shw,shh,mfd->samount,mfd->grad,mfd->spercent,mfd->fshift,shs,fa);
				}
			if (mfd->border)
				fa->dst.RectFill(rcMask->left-mfd->bordersize,
					rcMask->top-mfd->bordersize,
					rcMask->right - rcMask->left + mfd->bordersize*2,
					rcMask->bottom - rcMask->top + mfd->bordersize*2,
					RGB2PRGB(mfd->rgb));
			fa->dst.RectFill(rcMask->left,rcMask->top
				,rcMask->right - rcMask->left,rcMask->bottom - rcMask->top,0);
		}


	}

    return 0;
}


//****************************************
// EndProc - Done Filtering 
//
// Good place to clean up 
//****************************************
int ezMaskEndProc(FilterActivation *fa, const FilterFunctions *ff) {
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;


    return 0;
}

void deinitProc(FilterActivation *fa, const FilterFunctions *ff) {
	    MyFilterData *mfd = (MyFilterData *)fa->filter_data;

/* MARK - need to comment out to avoid crash */

//	AVIStreamGetFrameClose(mfd->pgf); // close the stream
//    if(mfd->pfile) AVIFileRelease(mfd->pfile);  // closes the file 
}


//****************************************
// ParamProc - Set the Frame size
//
// Good place to .....
//****************************************
long ezMaskParamProc(FilterActivation *fa, const FilterFunctions *ff) {
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	fa->dst.pitch = (fa->dst.w*4 + 7) & -8;

	mfd->dstW = fa->dst.w;
	mfd->dstH = fa->dst.h;


    return FILTERPARAM_SWAP_BUFFERS;
}

// colordisp/drawdimen added by MARK

void colordisp(HDC hdc,MyFilterData *mfd,int i)
{
SetBkMode(hdc,TRANSPARENT);
if (i)
	{
	SetTextColor(hdc,mfd->srgb);
	TextOut(hdc,174,370,"COLOR",5);
	}
else
	{
	SetTextColor(hdc,mfd->rgb);
	TextOut(hdc,130,300,"COLOR",5);
	}
}

void drawdimen(HWND Hwnd,RECT *rcMask,MyFilterData *mfd)
{
int w=0,h=0,t=0,l=0,r=0,b=0;
char str[256];

if (rcMask!=NULL)
	{
	t=(rcMask->top<rcMask->bottom)?max(rcMask->top,0):max(rcMask->bottom,0);
	l=(rcMask->left<rcMask->right)?max(rcMask->left,0):max(rcMask->right,0);
	r=(rcMask->left<rcMask->right)?min(rcMask->right,mfd->dstW):min(rcMask->left,mfd->dstW);
	b=(rcMask->top<rcMask->bottom)?min(rcMask->bottom,mfd->dstH):min(rcMask->top,mfd->dstH);
	w = (r-l)+1;
	h = (b-t)+1;
	}
SetDlgItemInt(Hwnd, IDC_TOP, t, FALSE);
SetDlgItemInt(Hwnd, IDC_LEFT, l, FALSE);
SetDlgItemInt(Hwnd, IDC_RIGHT, r, FALSE);
SetDlgItemInt(Hwnd, IDC_BOTTOM, b, FALSE);
sprintf(str,"%d x %d",w,h);
SetDlgItemText(Hwnd, IDC_WSIZE, str);
}

//****************************************
// ConfigDlgProc - Controls our configuration dialog
//
// Some additions by MARK
//
// This is where you need to do windows programing.
//****************************************
BOOL CALLBACK ConfigDlgProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam) {
    MyFilterData *mfd = (MyFilterData *)GetWindowLong(hdlg, DWL_USER);

	static	char msgDraw[] = "Left click and drag in the preview window to draw the mask.";
	static	char msgMove[] = "Left click on the mask, and drag to move it in the preview window.";
	char str[256],str1[20],str2[20];
	CHOOSECOLOR cc;
	COLORREF cols[16];
	int i,v;
	HDC hdc;
	HWND wbc;
	PAINTSTRUCT paints;

	//

    switch(msg) {
		case WM_PAINT:
			hdc=BeginPaint(hdlg,&paints);
			colordisp(hdc,mfd,0);
			colordisp(hdc,mfd,1);
			EndPaint(hdlg,&paints);
			return 0;
        case WM_INITDIALOG:
            SetWindowLong(hdlg, DWL_USER, lParam);
            mfd = (MyFilterData *)lParam;
			mfd->hdc2=GetDC(hdlg);
			if (!(mfd->brb))
				{
				mfd->rgb=RGB(255,255,255); mfd->bordersize=1; mfd->brb=1;
				mfd->shadow=0;
				mfd->samount=20;
				mfd->spercent=45;
				mfd->grad=3;
				mfd->fshift=7;
				mfd->srgb=RGB(0,0,0);
				}

            CheckDlgButton(hdlg, IDC_MOVE, mfd->mode?BST_CHECKED:BST_UNCHECKED);
            CheckDlgButton(hdlg, IDC_DRAW, !(mfd->mode)?BST_CHECKED:BST_UNCHECKED);
            CheckDlgButton(hdlg, IDC_INV, mfd->inverse?BST_CHECKED:BST_UNCHECKED);
            CheckDlgButton(hdlg, IDC_REPEAT, mfd->repeat?BST_CHECKED:BST_UNCHECKED);
            CheckDlgButton(hdlg, IDC_START, mfd->setstart?BST_CHECKED:BST_UNCHECKED);
            CheckDlgButton(hdlg, IDC_BORDER, mfd->border?BST_CHECKED:BST_UNCHECKED);
            CheckDlgButton(hdlg, IDC_MAINT, mfd->maintbp?BST_CHECKED:BST_UNCHECKED);
            CheckDlgButton(hdlg, IDC_ASPECT, mfd->aspect?BST_CHECKED:BST_UNCHECKED);
            CheckDlgButton(hdlg, IDC_BILINEAR, mfd->bilinear?BST_CHECKED:BST_UNCHECKED);
			SetDlgItemText(hdlg, IDC_INFO, (mfd->mode)?msgMove:msgDraw);

			SetDlgItemText(hdlg, IDC_FILE, mfd->szAviPath);
        	sprintf(str,"%d %d %d %d",mfd->rcMask.left,mfd->rcMask.top,mfd->rcMask.right,mfd->rcMask.bottom);
			SetDlgItemText(hdlg, IDC_COORDS, str);
			sprintf(str,"%d %d",mfd->startframe,mfd->endframe);
			SetDlgItemText(hdlg, IDC_STARTEND, str);
			SendDlgItemMessage(hdlg, IDC_STARTEND, EM_SETREADONLY,!mfd->setstart, 0);
			sprintf(str,"%d",mfd->bordersize);
			SetDlgItemText(hdlg, IDC_BORDERSIZE, str);
			SendDlgItemMessage(hdlg, IDC_BORDERSIZE, EM_SETREADONLY,!mfd->border, 0);
			wbc=GetDlgItem(hdlg, IDC_CCOLOR); EnableWindow(wbc,mfd->border);
			sprintf(str,"%d",mfd->frameoff);
			SetDlgItemText(hdlg, IDC_FRAMEO, str);
            CheckDlgButton(hdlg, IDC_REVERSE, mfd->reverse?BST_CHECKED:BST_UNCHECKED);
			SetDlgItemInt(hdlg, IDC_FRAMEC, mfd->length, FALSE);
			drawdimen(hdlg,&(mfd->rcMask),mfd);
			sprintf(str,"%d x %d",mfd->width,mfd->height);
			SetDlgItemText(hdlg, IDC_RESOL, str);
            CheckDlgButton(hdlg, IDC_SHADOW, mfd->shadow?BST_CHECKED:BST_UNCHECKED);
			sprintf(str,"%d",mfd->samount);
			SetDlgItemText(hdlg, IDC_SHADOWA, str);
			SendDlgItemMessage(hdlg, IDC_SHADOWA, EM_SETREADONLY,!mfd->shadow, 0);
			sprintf(str,"%d",mfd->spercent);
			SetDlgItemText(hdlg, IDC_PERCENT, str);
			SendDlgItemMessage(hdlg, IDC_PERCENT, EM_SETREADONLY,!mfd->shadow, 0);
			SendDlgItemMessage(hdlg, IDC_GRAD, CB_ADDSTRING,0,(long)"No gradient shadow");
			SendDlgItemMessage(hdlg, IDC_GRAD, CB_ADDSTRING,1,(long)"Gradient shadow, random var.");
			SendDlgItemMessage(hdlg, IDC_GRAD, CB_ADDSTRING,2,(long)"Gradient, no random variation");
			SendDlgItemMessage(hdlg, IDC_GRAD, CB_ADDSTRING,3,(long)"Gradient, rand. var., fixed shift");
			SendDlgItemMessage(hdlg, IDC_GRAD, CB_ADDSTRING,4,(long)"Gradient, no rand. var., fixed s.");
			SendDlgItemMessage(hdlg, IDC_GRAD, CB_ADDSTRING,5,(long)"No gradient, fixed shift");
			SendDlgItemMessage(hdlg, IDC_GRAD, CB_SETCURSEL,mfd->grad,0L);
			wbc=GetDlgItem(hdlg, IDC_GRAD); EnableWindow(wbc,mfd->shadow);
			sprintf(str,"%d",mfd->fshift);
			SetDlgItemText(hdlg, IDC_FSHIFT, str);
			if ((mfd->shadow==0)||(mfd->grad<=2)) { i=1; } else { i=0; }
			SendDlgItemMessage(hdlg, IDC_FSHIFT, EM_SETREADONLY,i, 0);
			wbc=GetDlgItem(hdlg, IDC_SCOLOR); EnableWindow(wbc,mfd->shadow);

			mfd->dlgHwnd = hdlg;
			
			if (mfd->ifp) {
				mfd->ifp->InitButton(GetDlgItem(hdlg, IDC_PREVIEW));
				mfd->ifp->Toggle(hdlg);
			}



            return TRUE;

        case WM_COMMAND:
            switch(LOWORD(wParam)) {
            case IDOK:
				mfd->mode = !!IsDlgButtonChecked(hdlg, IDC_MOVE);
                EndDialog(hdlg, 0);
                return TRUE;
            case IDCANCEL:
                EndDialog(hdlg, 1);
                return FALSE;
            case IDC_MOVE:
            case IDC_DRAW:
				mfd->mode = !!IsDlgButtonChecked(hdlg, IDC_MOVE);
				SetDlgItemText(hdlg, IDC_INFO, (mfd->mode)?msgMove:msgDraw);
				return TRUE;
            case IDC_INV:
				mfd->inverse = IsDlgButtonChecked(hdlg, IDC_INV);
				mfd->ifp->RedoFrame();
				return TRUE;
            case IDC_REPEAT:
				mfd->repeat = IsDlgButtonChecked(hdlg, IDC_REPEAT);
				mfd->ifp->RedoFrame();
				return TRUE;
            case IDC_MAINT:
				mfd->maintbp = IsDlgButtonChecked(hdlg, IDC_MAINT);
				mfd->ifp->RedoFrame();
				return TRUE;
            case IDC_ASPECT:
				mfd->aspect = IsDlgButtonChecked(hdlg, IDC_ASPECT);
				return TRUE;
            case IDC_BILINEAR:
				mfd->bilinear = IsDlgButtonChecked(hdlg, IDC_BILINEAR);
				mfd->ifp->RedoFrame();
				return TRUE;
            case IDC_START:
				mfd->setstart = IsDlgButtonChecked(hdlg, IDC_START);
				SendDlgItemMessage(hdlg, IDC_STARTEND, EM_SETREADONLY,!mfd->setstart, 0);
				mfd->ifp->RedoFrame();
				return TRUE;
            case IDC_BORDER:
				mfd->border = IsDlgButtonChecked(hdlg, IDC_BORDER);
				SendDlgItemMessage(hdlg, IDC_BORDERSIZE, EM_SETREADONLY,!mfd->border, 0);
				wbc=GetDlgItem(hdlg, IDC_CCOLOR); EnableWindow(wbc,mfd->border);
				mfd->ifp->RedoFrame();
				return TRUE;
			case IDC_PREVIEW:
				if (mfd->ifp)
					mfd->ifp->Toggle(hdlg);
				return TRUE;
			case IDC_COORDS:
				GetDlgItemText(hdlg,IDC_COORDS,str,256);
        		sscanf(str,"%d %d %s %s",&(mfd->rcMask.left),&(mfd->rcMask.top),str1,str2);
				if (str1[0]=='w') { sscanf(str1+1,"%d",&v); mfd->rcMask.right=mfd->rcMask.left+v-1; }
				else if (str1[0]=='h') { sscanf(str1+1,"%d",&v); mfd->rcMask.bottom=mfd->rcMask.top+v-1; }
				else { sscanf(str1,"%d",&(mfd->rcMask.right)); }
				if (str2[0]=='w') { sscanf(str2+1,"%d",&v); mfd->rcMask.right=mfd->rcMask.left+v-1; }
				else if (str2[0]=='h') { sscanf(str2+1,"%d",&v); mfd->rcMask.bottom=mfd->rcMask.top+v-1; }
				else { sscanf(str2,"%d",&(mfd->rcMask.bottom)); }
				drawdimen(hdlg,&(mfd->rcMask),mfd);
				mfd->ifp->RedoFrame();
				return TRUE;
			case IDC_STARTEND:
				GetDlgItemText(hdlg,IDC_STARTEND,str,256);
        		sscanf(str,"%d %d",&(mfd->startframe),&(mfd->endframe));
				mfd->ifp->RedoFrame();
				return TRUE;
			case IDC_BORDERSIZE:
				GetDlgItemText(hdlg,IDC_BORDERSIZE,str,256);
        		sscanf(str,"%d",&(mfd->bordersize));
				mfd->ifp->RedoFrame();
				return TRUE;
			case IDC_CCOLOR:
				cols[0]=mfd->rgb;
				for (i=1;i<16;i++) { cols[i]=0; }
				cc.lStructSize=sizeof(CHOOSECOLOR);
				cc.hwndOwner=hdlg;
				cc.hInstance=NULL;
				cc.rgbResult=mfd->rgb;
				cc.lpCustColors=cols;
				cc.Flags=CC_RGBINIT|CC_FULLOPEN;
				if (ChooseColor(&cc))
					{
					mfd->rgb=cols[0];
					colordisp(mfd->hdc2,mfd,0);
					mfd->ifp->RedoFrame();
					}
				return TRUE;
			case IDC_SCOLOR:
				cols[0]=mfd->srgb;
				for (i=1;i<16;i++) { cols[i]=0; }
				cc.lStructSize=sizeof(CHOOSECOLOR);
				cc.hwndOwner=hdlg;
				cc.hInstance=NULL;
				cc.rgbResult=mfd->srgb;
				cc.lpCustColors=cols;
				cc.Flags=CC_RGBINIT|CC_FULLOPEN;
				if (ChooseColor(&cc))
					{
					mfd->srgb=cols[0];
					colordisp(mfd->hdc2,mfd,1);
					mfd->ifp->RedoFrame();
					}
				return TRUE;
			case IDC_FRAMEO:
				GetDlgItemText(hdlg,IDC_FRAMEO,str,256);
        		sscanf(str,"%d",&(mfd->frameoff));
				mfd->ifp->RedoFrame();
				return TRUE;
            case IDC_REVERSE:
				mfd->reverse = IsDlgButtonChecked(hdlg, IDC_REVERSE);
				mfd->ifp->RedoFrame();
				return TRUE;
            case IDC_SHADOW:
				mfd->shadow = IsDlgButtonChecked(hdlg, IDC_SHADOW);
				SendDlgItemMessage(hdlg, IDC_SHADOWA, EM_SETREADONLY,!mfd->shadow, 0);
				SendDlgItemMessage(hdlg, IDC_PERCENT, EM_SETREADONLY,!mfd->shadow, 0);
				wbc=GetDlgItem(hdlg, IDC_GRAD); EnableWindow(wbc,mfd->shadow);
				if ((mfd->shadow==0)||(mfd->grad<=2)) { i=1; } else { i=0; }
				SendDlgItemMessage(hdlg, IDC_FSHIFT, EM_SETREADONLY,i, 0);
				wbc=GetDlgItem(hdlg, IDC_SCOLOR); EnableWindow(wbc,mfd->shadow);
				mfd->ifp->RedoFrame();
				return TRUE;
			case IDC_SHADOWA:
				GetDlgItemText(hdlg,IDC_SHADOWA,str,256);
        		sscanf(str,"%d",&(mfd->samount));
				mfd->ifp->RedoFrame();
				return TRUE;
			case IDC_PERCENT:
				GetDlgItemText(hdlg,IDC_PERCENT,str,256);
        		sscanf(str,"%d",&(mfd->spercent));
				mfd->ifp->RedoFrame();
				return TRUE;
			case IDC_GRAD:
				i=SendDlgItemMessage(hdlg,IDC_GRAD,CB_GETCURSEL,-1,0);
				if (i!=-1) { mfd->grad=i; }
				if ((mfd->shadow==0)||(mfd->grad<=2)) { i=1; } else { i=0; }
				SendDlgItemMessage(hdlg, IDC_FSHIFT, EM_SETREADONLY,i, 0);
				mfd->ifp->RedoFrame();
				return TRUE;
			case IDC_FSHIFT:
				GetDlgItemText(hdlg,IDC_FSHIFT,str,256);
        		sscanf(str,"%d",&(mfd->fshift));
				mfd->ifp->RedoFrame();
				return TRUE;

			case IDC_FILE_BROWSE:
				if (const char *fn = openFileDlg(hdlg, mfd->szAviPath)) {
					SetDlgItemText(hdlg, IDC_FILE, fn);
					strcpy(mfd->szAviPath, fn);
					testAVIFile(mfd->szAviPath, mfd);
					if (mfd->ifp) {mfd->ifp->RedoFrame();}
				}
				return TRUE;


            }
            break;
    }

    return FALSE;
}

//****************************************
// fPreviewButtonCallback - Vdub calls when preview is opened/closed
//
// Turn on/off capture of preview window mouse events
//****************************************

void *fPreviewButtonCallback(bool fNewState, void *pData) {

	MyFilterData *mfd = (MyFilterData *) pData;
	g_mfd = mfd;


	if(fNewState) {
		mfd->fpHwnd = FindWindow(0,"Filter Preview");

		//*** Install filter proc for preview windows messages ....
		mfd->fpWndProc = (WNDPROC) SetWindowLong(mfd->fpHwnd, GWL_WNDPROC, (LONG) nPreviewWndProc );

	} else {

		//*** restore proc for filter preview windows messages ....
		mfd->fpWndProc = (WNDPROC) SetWindowLong(mfd->fpHwnd, GWL_WNDPROC, (LONG) mfd->fpWndProc );
	}
	return FALSE;
}



//****************************************
// ConfigProc - Just before our dialog box is opened
//
// This is where you .....
//****************************************
int ezMaskConfigProc(FilterActivation *fa, const FilterFunctions *ff, HWND hwnd) {
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	//---- Setup preview w/ callback on toggle
	mfd->ifp = fa->ifp;
	mfd->ifp->SetButtonCallback((FilterPreviewButtonCallback)fPreviewButtonCallback, mfd); 


    return DialogBoxParam(fa->filter->module->hInstModule,
            MAKEINTRESOURCE(IDD_FILTER_EZMASK), hwnd,
            (DLGPROC) ConfigDlgProc, (LPARAM)fa->filter_data);
}


//****************************************
// StringProc - Called to get onfiguration settings you
// see in the filters config list.
//
// Basically builds a string of the current settings.
//****************************************
void ezMaskStringProc(const FilterActivation *fa, const FilterFunctions *ff, char *str) {
    //char *desc[50];
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;

    sprintf(str, " (%d,%d - %d,%d)",
        mfd->rcMask.left,
        mfd->rcMask.top,
        mfd->rcMask.right,
        mfd->rcMask.bottom);
}


//****************************************
// ScriptConfig - Sets your settings from a list
// of script arguments.
//****************************************
void ezMaskScriptConfig(IScriptInterpreter *isi, void *lpVoid, CScriptValue *argv, int argc) {
    FilterActivation *fa = (FilterActivation *)lpVoid;
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;

    mfd->rcMask.left    = argv[0].asInt();
    mfd->rcMask.top    = argv[1].asInt();
    mfd->rcMask.right    = argv[2].asInt();
    mfd->rcMask.bottom    = argv[3].asInt();
}

//****************************************
// FssProc - Saves your args as a string for scripts.
//****************************************
bool ezMaskFssProc(FilterActivation *fa, const FilterFunctions *ff, char *buf, int buflen) {
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;

    _snprintf(buf, buflen, "Config(%d,%d,%d,%d)",
        mfd->rcMask.left,
        mfd->rcMask.top,
        mfd->rcMask.right,
        mfd->rcMask.bottom);

    return true;
}

////////////////////////////////////////////
// Msg Proc to process filter preview windows messages
////////////////////////////////////////////

LRESULT CALLBACK nPreviewWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam) {

    POINTS ptLBD;
	POINT ptLBD2;
    RECT rcClient, *rcMask;                 // client area rectangle 
    POINT ptClientUL;              // client upper left corner 
    POINT ptClientLR;              // client lower right corner 
	char str[256];


	switch(msg) {

        case WM_MOUSEMOVE: 
 
            // When moving the mouse, the user must hold down 
            // the left mouse button to draw lines. 
 
            if (wParam & MK_LBUTTON)  { 
				if(g_mfd->drawing == TRUE) {
					rcMask = (RECT *) &(g_mfd->rcMask);

					ptLBD = MAKEPOINTS(lParam); 
					ptLBD.y = ptLBD.y -4;
					ptLBD.x = ptLBD.x -4;

					if(g_mfd->mode == 0) {
						rcMask->bottom = ptLBD.y;
						rcMask->right = ptLBD.x;
						if ((g_mfd->aspect)&&(g_mfd->ppavi)&&(g_mfd->width!=0))
							{
							double asp;

							asp=(double)(g_mfd->height)/(double)(g_mfd->width);
							rcMask->bottom=(int)((double)(rcMask->right-rcMask->left)*asp)+rcMask->top;
							}
					} else {
						OffsetRect(rcMask, ptLBD.x - g_mfd->ptsMove.x , ptLBD.y - g_mfd->ptsMove.y);
						g_mfd->ptsMove.x = ptLBD.x; g_mfd->ptsMove.y = ptLBD.y;
					}

					//**** The labels in the other dialog ****
					drawdimen(g_mfd->dlgHwnd,rcMask,g_mfd);

					g_mfd->ifp->RedoFrame();
				}
			}
			break;

       case WM_LBUTTONDOWN: 
 
		ptLBD = MAKEPOINTS(lParam); 
		POINTSTOPOINT(ptLBD2,ptLBD);

		if(g_mfd->mode==1) { //*** Move

			//** Did you click in the rectangle ?
			rcMask = (RECT *) &(g_mfd->rcMask);
			if(PtInRect(rcMask, ptLBD2) ){
				g_mfd->drawing = TRUE;
 
				GetClientRect(hWnd, &rcClient); 
				ptClientUL.x = rcClient.left + 4; 
				ptClientUL.y = rcClient.top + 4; 
				ptClientLR.x = ptClientUL.x + g_mfd->dstW; 
				ptClientLR.y = ptClientUL.y + g_mfd->dstH; 

				ClientToScreen(hWnd, &ptClientUL); 
				ClientToScreen(hWnd, &ptClientLR); 
 
				SetRect(&rcClient, ptClientUL.x, ptClientUL.y, ptClientLR.x, ptClientLR.y); 
				ClipCursor(&rcClient); 

				g_mfd->ptsMove.x = ptLBD.x; g_mfd->ptsMove.y = ptLBD.y;
			   
			}


		} else { //--- Draw Mode
		
			//**** If click in client area start draw 
			if(ptLBD.x-3 > 0 
				&& ptLBD.x-3 < g_mfd->dstW 
				&& ptLBD.y-3 > 0 
				&& ptLBD.y-3 < g_mfd->dstH) {

				g_mfd->drawing = TRUE;

				GetClientRect(hWnd, &rcClient); 
				ptClientUL.x = rcClient.left + 4; 
				ptClientUL.y = rcClient.top + 4; 
				ptClientLR.x = ptClientUL.x + g_mfd->dstW; 
				ptClientLR.y = ptClientUL.y + g_mfd->dstH; 

				ClientToScreen(hWnd, &ptClientUL); 
				ClientToScreen(hWnd, &ptClientLR); 
 
				SetRect(&rcClient, ptClientUL.x, ptClientUL.y, ptClientLR.x, ptClientLR.y); 
				ClipCursor(&rcClient); 


				rcMask = (RECT *) &(g_mfd->rcMask);

				//**** Set Rect to 1x1 at click point
				SetRect(rcMask, ptLBD.x -4, ptLBD.y -4, ptLBD.x -3, ptLBD.y -3);
			}
		}
		//g_mfd->frame = g_mfd->saveFrame; 
			
		break;

       case WM_LBUTTONUP: 


		   if(g_mfd->drawing) {

				rcMask = (RECT *) &(g_mfd->rcMask);
			   if(g_mfd->mode == 0) {

					rcClient.bottom = (rcMask->top < rcMask->bottom)?rcMask->bottom:rcMask->top;
					rcClient.top = (rcMask->top > rcMask->bottom)?rcMask->bottom:rcMask->top;
					rcClient.right = (rcMask->left > rcMask->right)?rcMask->left:rcMask->right;
					rcClient.left = (rcMask->left < rcMask->right)?rcMask->left:rcMask->right;

					//*** Make sure UL is before LR
					rcMask->bottom = rcClient.bottom;
					rcMask->top = rcClient.top;
					rcMask->right = min(rcClient.right,g_mfd->dstW - 1);
					rcMask->left = max(rcClient.left,0);

			   } else {
					//*** Make sure all points are on screen
					rcMask->bottom = min(rcMask->bottom, g_mfd->dstH - 1);
					rcMask->top = max(rcMask->top,0);
					rcMask->right = min(rcMask->right,g_mfd->dstW - 1);
					rcMask->left = max(rcMask->left,0);
			   }
        		sprintf(str,"%d %d %d %d",rcMask->left,rcMask->top,rcMask->right,rcMask->bottom);
				SetDlgItemText(g_mfd->dlgHwnd, IDC_COORDS, str);

				ClipCursor(NULL); 
				g_mfd->drawing = FALSE;
				g_mfd->ifp->RedoFrame();
		   }

			//default:
	}

	return CallWindowProc( g_mfd->fpWndProc, hWnd, msg, wParam, lParam);
//	return CallWindowProc((DLGPROC) g_mfd->fpWndProc, hWnd, msg, wParam, lParam);


}


//*********************************

void xRectFill(const FilterActivation *fa, PixCoord x, PixCoord y, PixDim dx, PixDim dy, Pixel32 c) {


	// Do the blit

	Pixel32 *dst = (Pixel32 *)fa->dst.data;
	int w = fa->dst.w;
	int h = fa->dst.h;
	int m = fa->dst.modulo;
	PixOffset pitch = fa->dst.pitch;



	if (dx == -1) dx = w;
	if (dy == -1) dy = h;

	// clip to destination bitmap

	if (x < 0) { dx+=x; x=0; }
	if (y < 0) { dy+=y; y=0; }
	if (x+dx > w) dx=w-x;
	if (y+dy > h) dy=h-y;

	// anything left to fill?

	if (dx<=0 || dy<=0) return;

	// compute coordinates

	dst = fa->dst.Address32(x, y+dy-1);

	// do the fill

	do {
		PixDim dxt = dx;
		Pixel32 *dst2 = dst;

		do {
			*dst2++ = c;
		} while(--dxt);

		dst = (Pixel32 *)((char *)dst + pitch);
	} while(--dy);

return;

}

static const char *openFileDlg(HWND hwnd, const char *oldfn) {
	OPENFILENAME ofn;
	static char szFile[MAX_PATH];
	char szFileTitle[MAX_PATH];

	///////////////

	if (oldfn)
		strcpy(szFile, oldfn);

	szFileTitle[0]=0;

	ofn.lStructSize			= sizeof (OPENFILENAME) ;
	ofn.hwndOwner			= hwnd;
	ofn.lpstrFilter			= "AVI file (*.avi,*.avs)\0*.avi;*.avs\0All files (*.*)\0*.*\0";
	ofn.lpstrCustomFilter	= NULL;
	ofn.nFilterIndex		= 1;
	ofn.lpstrFile			= szFile;
	ofn.nMaxFile			= sizeof szFile;
	ofn.lpstrFileTitle		= szFileTitle;
	ofn.nMaxFileTitle		= sizeof szFileTitle;
	ofn.lpstrInitialDir		= NULL;
	ofn.lpstrTitle			= "Select AVI";
	ofn.Flags				= OFN_EXPLORER | OFN_PATHMUSTEXIST | OFN_HIDEREADONLY | OFN_ENABLESIZING;
	ofn.lpstrDefExt			= NULL;

	if (GetOpenFileName(&ofn))
		return szFile;

	return NULL;
}

// testAVIFile: Some additions by MARK

void testAVIFile(char * filename, MyFilterData *mfd) {


  LONG hr; //--- Return code from file open.  0 is good 
  AVIFILEINFO pfi; //--- Structure to hold File Info 
  AVISTREAMINFO psi; //--- Structure to hold Stream Info (where the fccHandler is) 
  HRESULT hresult; 
  long bmiSize;
  BITMAPINFOHEADER *bmih,*lpbih;
  char buf[20];
  
     
 

//---- Open the file.  If you don't get a zero back, Tell avs there  was an error 
  hr = AVIFileOpen(&mfd->pfile, filename, OF_READ, NULL); 
  if (hr != 0){ 
      return; 
  } 
  

//---- Get the File Info into our local structure so we can look 
   AVIFileInfo(mfd->pfile, &pfi, sizeof(pfi)); 


    //    if (AVIFileGetStream(pfile, &ppavi, NULL, streamtypeVIDEO) != AVIERR_OK) 
   AVIFileGetStream(mfd->pfile, &mfd->ppavi, streamtypeVIDEO, 0);
    //    break; 
  
      if (mfd->ppavi != NULL)  { 
         hresult = AVIStreamInfo(mfd->ppavi, &psi, sizeof(psi)); 

         sprintf(buf,"%c%c%c%c",psi.fccHandler, psi.fccHandler>>8, psi.fccHandler>>16, psi.fccHandler>>24); 
         sprintf(buf,"%c%c%c%c", psi.fccType , psi.fccType>>8, psi.fccType>>16, psi.fccType>>24); 

		 if(AVIStreamReadFormat(mfd->ppavi,AVIStreamStart(mfd->ppavi),NULL, &bmiSize))
			 return;

		 LPBYTE pChunk = new BYTE[bmiSize];
		 if(!pChunk)
			 return; //--- Allocation Error

		 if(AVIStreamReadFormat(mfd->ppavi,AVIStreamStart(mfd->ppavi),pChunk, &bmiSize))
			 return;

		 bmih = (LPBITMAPINFOHEADER) &pChunk;
  		 mfd->pgf = AVIStreamGetFrameOpen(mfd->ppavi,NULL);
		 lpbih = (LPBITMAPINFOHEADER) AVIStreamGetFrame(mfd->pgf, 0);
		 if (lpbih)
			{
			mfd->width = lpbih->biWidth;
			mfd->height = lpbih->biHeight;
			}
		 mfd->length=AVIStreamLength(mfd->ppavi);
		 SetDlgItemInt(mfd->dlgHwnd, IDC_FRAMEC, mfd->length, FALSE);
		 sprintf(buf,"%d x %d",mfd->width,mfd->height);
		 SetDlgItemText(mfd->dlgHwnd, IDC_RESOL, buf);

	  }


} 
